@@ -1,4 +1,4 @@
-# $Id: Builder.pm 603 2006-09-23 21:47:38Z pajas $
+# $Id: Builder.pm 772 2009-01-23 21:42:09Z pajas $
 
 package XML::LibXML::SAX::Builder;
 
@@ -7,7 +7,11 @@ use XML::NamespaceSupport;
 
 use vars qw ($VERSION);
 
-$VERSION = "1.61"; # VERSION TEMPLATE: DO NOT CHANGE
+sub CLONE_SKIP {
+  return $XML::LibXML::__threads_shared ? 0 : 1;
+}
+
+$VERSION = "1.69_1"; # VERSION TEMPLATE: DO NOT CHANGE
 
 sub new {
     my $class = shift;
@@ -33,9 +37,19 @@ sub done {
 sub set_document_locator {
 }
 
+sub start_dtd {
+  my ($self, $dtd) = @_;
+  if (defined $dtd->{Name} and
+      (defined $dtd->{SystemId} or defined $dtd->{PublicId})) {
+    $self->{DOM}->createExternalSubset($dtd->{Name},$dtd->{PublicId},$dtd->{SystemId});
+  }
+}
+
+sub end_dtd {
+}
+
 sub start_document {
     my ($self, $doc) = @_;
-
     $self->{DOM} = XML::LibXML::Document->createDocument();
 
     if ( defined $self->{Encoding} ) {
