@@ -1,4 +1,4 @@
-# $Id: Generator.pm 603 2006-09-23 21:47:38Z pajas $
+# $Id: Generator.pm 772 2009-01-23 21:42:09Z pajas $
 
 package XML::LibXML::SAX::Generator;
 
@@ -7,7 +7,11 @@ use strict;
 use XML::LibXML;
 use vars qw ($VERSION);
 
-$VERSION = "1.61"; # VERSION TEMPLATE: DO NOT CHANGE
+$VERSION = "1.69_1"; # VERSION TEMPLATE: DO NOT CHANGE
+
+sub CLONE_SKIP {
+  return $XML::LibXML::__threads_shared ? 0 : 1;
+}
 
 warn("This class (", __PACKAGE__, ") is deprecated!");
 
