@@ -1,4 +1,4 @@
-# $Id: Parser.pm,v 1.21 2003/05/22 23:23:44 phish Exp $
+# $Id: Parser.pm 772 2009-01-23 21:42:09Z pajas $
 
 package XML::LibXML::SAX::Parser;
 
@@ -10,9 +10,13 @@ use XML::LibXML::Common qw(:libxml);
 use XML::SAX::Base;
 use XML::SAX::DocumentLocator;
 
-$VERSION = '1.50';
+$VERSION = "1.69_1"; # VERSION TEMPLATE: DO NOT CHANGE
 @ISA = ('XML::SAX::Base');
 
+sub CLONE_SKIP {
+  return $XML::LibXML::__threads_shared ? 0 : 1;
+}
+
 sub _parse_characterstream {
     my ($self, $fh, $options) = @_;
     die "parsing a characterstream is not supported at this time";
@@ -43,6 +47,36 @@ sub generate {
     my $self = shift;
     my ($node) = @_;
 
+    my $doc = $node->ownerDocument();
+    {
+      # precompute some DocumentLocator values
+      my %locator = (
+	PublicId => undef,
+	SystemId => undef,
+	Encoding => undef,
+	XMLVersion => undef,
+       );
+      my $dtd = defined $doc ? $doc->externalSubset() : undef;
+      if (defined $dtd) {
+	$locator{PublicId} = $dtd->publicId();
+	$locator{SystemId} = $dtd->systemId();
+      }
+      if (defined $doc) {
+	$locator{Encoding} = $doc->encoding();
+	$locator{XMLVersion} = $doc->version();
+      }
+      $self->set_document_locator(
+	XML::SAX::DocumentLocator->new(
+	  sub { $locator{PublicId} },
+	  sub { $locator{SystemId} },
+	  sub { defined($self->{current_node}) ? $self->{current_node}->line_number() : undef },
+	  sub { 1 },
+	  sub { $locator{Encoding} },
+	  sub { $locator{XMLVersion} },
+	 ),
+       );
+    }
+
     if ( $node->nodeType() == XML_DOCUMENT_NODE
          || $node->nodeType == XML_HTML_DOCUMENT_NODE ) {
         $self->start_document({});
@@ -55,6 +89,8 @@ sub generate {
 sub process_node {
     my ($self, $node) = @_;
 
+    local $self->{current_node} = $node;
+
     my $node_type = $node->nodeType();
     if ($node_type == XML_COMMENT_NODE) {
         $self->comment( { Data => $node->getData } );
@@ -102,6 +138,7 @@ sub process_node {
     else {
         # warn("unsupported node type: $node_type");
     }
+
 }
 
 sub process_element {
@@ -160,7 +197,7 @@ sub process_element {
         }
         else {
             my $ns = $attr->namespaceURI;
-            my $ns = $attr->namespaceURI;
+
             $ns = '' unless defined $ns;
             $key = "{$ns}".$attr->localname;
             ## Not sure why, but $attr->name is coming through stripped
@@ -219,52 +256,3 @@ sub process_element {
 1;
 
 __END__
-
-=head1 NAME
-
-XML::LibXML::SAX::Parser - LibXML DOM based SAX Parser
-
-=head1 SYNOPSIS
-
-  my $handler = MySAXHandler->new();
-  my $parser = XML::LibXML::SAX::Parser->new(Handler => $handler);
-  $parser->parse_uri("foo.xml");
-
-=head1 DESCRIPTION
-
-This class allows you to generate SAX2 events using LibXML. Note
-that this is B<not> a stream based parser, instead it parses
-documents into a DOM and traverses the DOM tree. The reason being
-that libxml2's stream based parsing is extremely primitive,
-and would require an extreme amount of work to allow SAX2
-parsing in a stream manner.
-
-=head1 WARNING
-
-WARNING WARNING WARNING
-
-This is NOT a streaming SAX parser. As I said above, this parser
-reads the entire document into a DOM and serialises it. Some
-people couldn't read that in the paragraph above so I've added
-this warning.
-
-There are many reasons, but if you want to write a proper SAX
-parser using the libxml2 library, please feel free and send it
-along to me.
-
-=head1 API
-
-The API is exactly the same as any other Perl SAX2 parser. See
-L<XML::SAX::Intro> for details.
-
-Aside from the regular parsing methods, you can access the
-DOM tree traverser directly, using the generate() method:
-
-  my $parser = XML::LibXML::SAX::Parser->new(...);
-  $parser->generate($dom_tree);
-
-This is useful for serializing DOM trees, for example that
-you might have done prior processing on, or that you have
-as a result of XSLT processing.
-
-=cut
