@@ -5,19 +5,22 @@ package XML::LibXML::XPathContext;
 use strict;
 use vars qw($VERSION @ISA $USE_LIBXML_DATA_TYPES);
 
+use Carp;
 use XML::LibXML;
 use XML::LibXML::NodeList;
 
-$VERSION = "1.61"; # VERSION TEMPLATE: DO NOT CHANGE
+$VERSION = "1.69_1"; # VERSION TEMPLATE: DO NOT CHANGE
 
 # should LibXML XPath data types be used for simple objects
 # when passing parameters to extension functions (default: no)
 $USE_LIBXML_DATA_TYPES = 0;
 
+sub CLONE_SKIP { 1 }
+
 sub findnodes {
     my ($self, $xpath, $node) = @_;
 
-    my @nodes = $self->_guarded_find_call('_findnodes', $xpath, $node);
+    my @nodes = $self->_guarded_find_call('_findnodes', $node, $xpath);
 
     if (wantarray) {
         return @nodes;
@@ -30,7 +33,7 @@ sub findnodes {
 sub find {
     my ($self, $xpath, $node) = @_;
 
-    my ($type, @params) = $self->_guarded_find_call('_find', $xpath, $node);
+    my ($type, @params) = $self->_guarded_find_call('_find', $node, $xpath,0);
 
     if ($type) {
         return $type->new(@params);
@@ -38,13 +41,19 @@ sub find {
     return undef;
 }
 
+sub exists {
+    my ($self, $xpath, $node) = @_;
+    my (undef, $value) = $self->_guarded_find_call('_find', $node, $xpath,1);
+    return $value;
+}
+
 sub findvalue {
     my $self = shift;
     return $self->find(@_)->to_literal->value;
 }
 
 sub _guarded_find_call {
-    my ($self, $method, $xpath, $node) = @_;
+    my ($self, $method, $node)=(shift,shift,shift);
 
     my $prev_node;
     if (ref($node)) {
@@ -53,12 +62,16 @@ sub _guarded_find_call {
     }
     my @ret;
     eval {
-        @ret = $self->$method($xpath);
+        @ret = $self->$method(@_);
     };
     $self->_free_node_pool;
     $self->setContextNode($prev_node) if ref($node);
 
-    if ($@) { die "ERROR: $@"; }
+    if ($@) { 
+      my $err = $@;
+      chomp $err;
+      croak $err; 
+    }
 
     return @ret;
 }
